@@ -1,3 +1,6 @@
+"""
+Copyright 2020 The Microsoft DeepSpeed Team
+"""
 from .cpu_adam import CPUAdamBuilder
 from .fused_adam import FusedAdamBuilder
 from .fused_lamb import FusedLambBuilder
@@ -5,6 +8,7 @@ from .sparse_attn import SparseAttnBuilder
 from .transformer import TransformerBuilder
 from .stochastic_transformer import StochasticTransformerBuilder
 from .utils import UtilsBuilder
+from .async_io import AsyncIOBuilder
 from .builder import get_default_compute_capatabilities
 
 # TODO: infer this list instead of hard coded
@@ -16,6 +20,7 @@ __op_builders__ = [
     SparseAttnBuilder(),
     TransformerBuilder(),
     StochasticTransformerBuilder(),
-    UtilsBuilder()
+    UtilsBuilder(),
+    AsyncIOBuilder()
 ]
 ALL_OPS = {op.name: op for op in __op_builders__}
